package rainmelt;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
public final class Account {
	
	public static File dataDirectory = new File("data" + File.separator + "account");
	private static final String HASH_ALGORITHM = "SHA-512";

	@XmlElement private String id;
	@XmlElement private byte[] passwordDigest;
	
	@SuppressWarnings("unused")
	private Account() {
	}
	
	public Account(final String id, final String password) {
		this.id = id;
		setPassword(password);
	}
	
	public String getID() {
		return id;
	}
	
	public void save() {
		synchronized (AccountManager.INSTANCE) {
			final File file = new File(dataDirectory, id);
			BufferedOutputStream out;
			try {
				out = new BufferedOutputStream(new FileOutputStream(file));
				JAXBContext c = JAXBContext.newInstance(this.getClass());
				Marshaller m = c.createMarshaller();
				m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
				m.marshal(this, out);
				out.close();
			} catch (JAXBException e) {
				e.printStackTrace();
			} catch (final IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	public synchronized void setPassword(final String password) {
		byte[] b;
		try {
			b = password.getBytes("UTF-8");
		} catch (final UnsupportedEncodingException e) {
			e.printStackTrace();
			return;
		}
		final MessageDigest md;
		try {
			md = MessageDigest.getInstance(HASH_ALGORITHM);
		} catch (final NoSuchAlgorithmException e) {
			e.printStackTrace();
			return;
		}
		synchronized (md) {
			md.reset();
			passwordDigest = md.digest(b);
		}
	}
	
	public synchronized boolean passwordMatches(final String password) {
		if (this.passwordDigest == null) {
			return false;
		}
		if (password == null) {
			return false;
		}
		
		byte[] b;
		try {
			b = password.getBytes("UTF-8");
		} catch (final UnsupportedEncodingException e) {
			e.printStackTrace();
			return false;
		}
		final MessageDigest md;
		try {
			md = MessageDigest.getInstance(HASH_ALGORITHM);
		} catch (final NoSuchAlgorithmException e) {
			e.printStackTrace();
			return false;
		}
		final byte[] digest;
		synchronized (md) {
			md.reset();
			digest = md.digest(b);
		}

		if (digest.length != passwordDigest.length) return false;

		for (int i=0; i<digest.length; i++) {
			if (digest[i] != passwordDigest[i]) return false;
		}

		return true;
	}
	
}
