package rainmelt;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.lang.ref.WeakReference;
import java.util.Map;
import java.util.WeakHashMap;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

public enum AccountManager {

	INSTANCE;
	
	private Map<String, WeakReference<Account>> map = new WeakHashMap<String, WeakReference<Account>>();
	
	public static final String ANONYMOUS_ID = "anonymous";
	
	static {
		if (!Account.dataDirectory.exists()) {
			if (!Account.dataDirectory.mkdirs()) {
				System.out.println("Could't create the account directory: " + Account.dataDirectory);
			}
		}
		
		Account account = INSTANCE.get(ANONYMOUS_ID);
		if (account == null) {
			account = new Account(ANONYMOUS_ID, "");
			account.save();
		}
	}
	
	public synchronized Account get(final String id) {
		final WeakReference<Account> reference = map.get(id);
		if (reference == null) {
			return loadAndPut(id);
		} else {
			final Account a = reference.get();
			if (a != null) {
				return a;
			} else {
				return loadAndPut(id);
			}
		}
	}
	
	private Account loadAndPut(final String id) {
		final Account account = load(id);
		if (account == null) {
			return null;
		} else {
			map.put(account.getID(), new WeakReference<Account>(account));
			return account;
		}
	}
	
	private Account load(final String id) {
		final File file = new File(Account.dataDirectory, id);
		final BufferedInputStream in;
		try {
			in = new BufferedInputStream(new FileInputStream(file));
		} catch (final FileNotFoundException e) {
			return null;
		}
		final Account account;
		try {
			JAXBContext c = JAXBContext.newInstance(Account.class);
			Unmarshaller u = c.createUnmarshaller();
			account = (Account)u.unmarshal(in);
		} catch (final JAXBException e) {
			e.printStackTrace();
			return null;
		}
		return account;
	}
	
	public synchronized Account create(final String id, final String password) {
		Account account = get(id);
		if (account != null) {
			return null;
		}
		account = new Account(id, password);
		map.put(account.getID(), new WeakReference<Account>(account));
		return account;
	}
	
	public boolean isValidID(final String id) {
		if (id.length() < 1) {
			return false;
		}
		for (int i = 0; i < id.length(); i++) {
			if (!Character.isLetter(id.charAt(i))) {
				return false;
			}
		}
		return true;
	}
	
}
