package rainmelt;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;

public final class FileSystem {
	
	public static final String separator = "/";
	public static final File defaultRootDirectory = new File("data" + File.separator + "fs");
	
	static {
		if (!defaultRootDirectory.exists()) {
			if (!defaultRootDirectory.mkdirs()) {
				System.out.println("Could'nt create the default root directory: " + defaultRootDirectory);
			}
		}
	}
	
	private final File rootDirectory;
	
	public FileSystem() {
		this(defaultRootDirectory);
	}

	public FileSystem(final File rootDirectory) {
		this.rootDirectory = rootDirectory;
	}
	
	public static String toCanonicalPath(final String path) {
		final boolean absolute = path.startsWith(separator);
		final String[] elements = path.split(Matcher.quoteReplacement(separator));
		final List<String> canonicalElements = new ArrayList<String>();
		for (String element : elements) {
			if (element.equals("")) {
			} else if (element.equals(".")) {
			} else if (element.equals("..")) {
				if (canonicalElements.size() > 0) {
					canonicalElements.remove(canonicalElements.size() - 1);
				}
			} else {
				canonicalElements.add(element);
			}
		}
		if (absolute) {
			return separator + StringUtil.join(canonicalElements, separator);
		} else {
			return StringUtil.join(canonicalElements, separator);
		}
	}
	
	public static String toAbsolutePath(final String currentPath, final String path) {
		if (path.startsWith(separator)) {
			return toCanonicalPath(path);
		} else {
			return toCanonicalPath(currentPath + separator + path);
		}
	}
	
	public File toFile(final String path) {
		final String absolutePath = toAbsolutePath(separator, path);
		return new File(rootDirectory, absolutePath.replaceAll(Matcher.quoteReplacement(separator), File.separator));
	}
	
}
