package rainmelt;

import java.net.InetSocketAddress;
import java.net.UnknownHostException;

import org.java_websocket.WebSocket;
import org.java_websocket.handshake.ClientHandshake;
import org.java_websocket.server.WebSocketServer;

public final class Server extends WebSocketServer {
	
	public Server(final int port) throws UnknownHostException {
		super(new InetSocketAddress(port));
	}

	public static void main(final String[] args) throws UnknownHostException {
		//WebSocket.DEBUG = true;
		int port;
		if (args.length >= 1) {
			port = Integer.parseInt(args[0]);
		} else {
			port = 2413;
		}
		final Server server = new Server(port);
		server.start();
		System.out.println("Server started on port " + server.getPort());
	}

	@Override
	public void onClose(final WebSocket conn, final int code, final String reason, final boolean remote) {
		HandlerManager.INSTANCE.remove(conn);
	}

	@Override
	public void onError(final WebSocket conn, final Exception ex) {
		ex.printStackTrace();
	}

	@Override
	public void onMessage(final WebSocket conn, final String message) {
		final Handler handler = HandlerManager.INSTANCE.get(conn);
		String resourceDescriptor = handler.getHandshake().getResourceDescriptor();
		if (resourceDescriptor.equals("/echo")) {
			conn.send(message);
		} else if (resourceDescriptor.equals("/terminal")) {
			handler.handle(message);
		}
	}

	@Override
	public void onOpen(final WebSocket conn, final ClientHandshake handshake) {
		final Handler handler = new Handler(conn, handshake);
		HandlerManager.INSTANCE.put(conn, handler);
	}

}
