package rainmelt.api;

import rainmelt.Account;
import rainmelt.AccountManager;
import rainmelt.Handler;
import rainmelt.packet.APIPacket;
import rainmelt.packet.ResultPacket;

public final class login implements API {

	@Override
	public void execute(final Handler handler, final APIPacket packet) {
		if (packet.args.length < 2) {
			final String m = "Missing operand\n";
			final ResultPacket r = new ResultPacket(packet.id, ResultPacket.Status.FAILURE, m);
			handler.send(r);
			return;
		}
		
		boolean created = false;
		Account account;
		synchronized (AccountManager.INSTANCE) {
			if (!AccountManager.INSTANCE.isValidID(packet.args[0])) {
				final String m = "Invalid ID: " + packet.args[0] + "\n";
				final ResultPacket r = new ResultPacket(packet.id, ResultPacket.Status.FAILURE, m);
				handler.send(r);
				return;
			}
			account = AccountManager.INSTANCE.get(packet.args[0]);
			if (account == null) {
				account = AccountManager.INSTANCE.create(packet.args[0], packet.args[1]);
				account.save();
				created = true;
			}
		}
		
		if (!account.passwordMatches(packet.args[1])) {
			final String m = "Login failed\n";
			final ResultPacket r = new ResultPacket(packet.id, ResultPacket.Status.FAILURE, m);
			handler.send(r);
			return;
		}
			
		handler.setAccount(account);
		
		if (created) {
			String m = "Created new account: " + account.getID() + "\n";
			m += account.getID() + " logged in\n";
			final ResultPacket r = new ResultPacket(packet.id, ResultPacket.Status.SUCCESS, m);
			handler.send(r);
		} else {
			final String m = account.getID() + " logged in\n";
			final ResultPacket r = new ResultPacket(packet.id, ResultPacket.Status.SUCCESS, m);
			handler.send(r);
		}
	}

}
