package rainmelt.fileapi;

import java.io.File;

import rainmelt.Handler;
import rainmelt.packet.FileAPIPacket;
import rainmelt.packet.FileExceptionPacket;
import rainmelt.packet.FileListPacket;

public final class list implements FileAPI {

	@Override
	public void execute(final Handler handler, final FileAPIPacket packet) {
		if (packet.args.length < 1) {
			final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.TOO_FEW_ARGUMENTS);
			handler.send(p);
			return;
		}
		
		final File file = handler.getFileSystem().toFile(packet.args[0]);
		
		if (!file.exists()) {
			final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.NOT_EXIST);
			handler.send(p);
		} else if (file.isFile()) {
			final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.IS_FILE);
			handler.send(p);
		} else if (file.isDirectory()) {
			final File[] files = file.listFiles();
			final FileListPacket p = new FileListPacket(packet.id);
			for (File f : files) {
				if (f.isDirectory()) {
					p.add(FileListPacket.Type.DIRECTORY, f.getName());
				} else if (f.isFile()) {
					p.add(FileListPacket.Type.FILE, f.getName());
				} else {
					p.add(FileListPacket.Type.OTHER, f.getName());
				}
			}
			handler.send(p);
		} else {
			final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.UNKNOWN_TYPE);
			handler.send(p);
		}
	}

}
