package rainmelt.fileapi;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import rainmelt.Handler;
import rainmelt.packet.FileAPIPacket;
import rainmelt.packet.FileExceptionPacket;
import rainmelt.packet.FilePacket;

public final class load implements FileAPI {

	@Override
	public void execute(final Handler handler, final FileAPIPacket packet) {
		if (packet.args.length < 1) {
			final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.TOO_FEW_ARGUMENTS);
			handler.send(p);
			return;
		}
		
		final File file = handler.getFileSystem().toFile(packet.args[0]);
		
		if (!file.exists()) {
			final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.NOT_EXIST);
			handler.send(p);
		} else if (file.isFile()) {
			final BufferedInputStream in;
			try {
				in = new BufferedInputStream(new FileInputStream(file));
			} catch (final FileNotFoundException e) {
				e.printStackTrace();
				final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.IO_ERROR);
				handler.send(p);
				return;
			}
			
			final int length = (int)file.length();
			final byte[] b = new byte[length];
			for (int i = 0; i < length; ) {
				int l;
				try {
					l = in.read(b, i, length - i);
				} catch (final IOException e) {
					e.printStackTrace();
					final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.IO_ERROR);
					handler.send(p);
					return;
				}
				if (l == -1) {
					break;
				}
				i += l;
			}
			
			try {
				in.close();
			} catch (final IOException e) {
				e.printStackTrace();
				final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.IO_ERROR);
				handler.send(p);
				return;
			}
			
			final FilePacket p = new FilePacket(packet.id, b);
			handler.send(p);
		} else if (file.isDirectory()) {
			final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.IS_DIRECTORY);
			handler.send(p);
		} else {
			final FileExceptionPacket p = new FileExceptionPacket(packet.id, FileExceptionPacket.Type.UNKNOWN_TYPE);
			handler.send(p);
		}
	}

}